import { Node } from 'estree';
export declare type EnterFunction = (this: {
    skip(): void;
}, node: Node, parent: Node | null, prop: string | undefined, index: number | undefined) => void;
export declare type LeaveFunction = (this: null, node: Node, parent: Node | null, prop: string | undefined, index: number | undefined) => void;
export interface WalkOptions {
    /**
     * A callback function that is called when a node is entered.  `this.skip()` will cause the walker to not
     * descend further into the tree.
     */
    enter?: EnterFunction;
    /**
     * A callback function that is called whenever a node is _exited_.
     */
    leave?: LeaveFunction;
}
/**
 * Walk a estree compatabile AST tree, calling any callback functions provided.
 * @param node The root estree Node to start walking
 * @param param1 An object with `enter` and `leave` properties
 */
export default function walk(node: Node, {enter, leave}: WalkOptions): void;
