(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Local context for enter callbacks that provides an ability to skip a node and its children.
     */
    const context = {
        skip() {
            context.shouldSkip = true;
        },
        shouldSkip: false
    };
    const childKeysCache = {};
    function visit(node, parent, enter, leave, prop, index) {
        if (!node) {
            return;
        }
        if (enter) {
            context.shouldSkip = false;
            enter.call(context, node, parent, prop, index);
            if (context.shouldSkip) {
                return;
            }
        }
        const keys = childKeysCache[node.type] ||
            (childKeysCache[node.type] = Object.keys(node).filter((key) => typeof node[key] === 'object'));
        keys.forEach((key) => {
            const value = node[key];
            if (Array.isArray(value)) {
                value.forEach((element, idx) => {
                    visit(element, node, enter, leave, key, idx);
                });
            }
            else if (value && value.type) {
                visit(value, node, enter, leave, key);
            }
        });
        if (leave) {
            leave.call(null, node, parent, prop, index);
        }
    }
    /**
     * Walk a estree compatabile AST tree, calling any callback functions provided.
     * @param node The root estree Node to start walking
     * @param param1 An object with `enter` and `leave` properties
     */
    function walk(node, { enter, leave }) {
        visit(node, null, enter, leave);
    }
    exports.default = walk;
});
//# sourceMappingURL=walk.js.map